#ifndef ROBOJAX_SHUNT_CURRENT_H
#define ROBOJAX_SHUNT_CURRENT_H

#include <inttypes.h>
#include <Print.h>

//energy meter values
#define CURRENT_POSITION_ROW 1
#define CURRENT_TITLE_POSITION_CHAR 0
#define CURRENT_TITLE "I:"
#define CURRENT_TITLE2 "Current: "
#define CURRENT_VALUE_POSITION_CHAR 2
#define CURRENT_VALUE2_POSITION_CHAR 9
#define CURRENT_UNIT_POSITION_CHAR 8
#define CURRENT_UNIT "A"


#define VOLTAGE_POSITION_ROW 1
#define VOLTAGE_TITLE_POSITION_CHAR 10
#define VOLTAGE_TITLE "V:"
#define VOLTAGE_VALUE_POSITION_CHAR 12
#define VOLTAGE_UNIT_POSITION_CHAR 18
#define VOLTAGE_UNIT "V"


#define POWER_POSITION_ROW 2
#define POWER_TITLE_POSITION_CHAR 0
#define POWER_TITLE "P:"
#define POWER_VALUE_POSITION_CHAR 2
#define POWER_UNIT_POSITION_CHAR 8
#define POWER_UNIT "W"

#define ENERGY_POSITION_ROW 2
#define ENERGY_TITLE_POSITION_CHAR 10
#define ENERGY_TITLE "E:"
#define ENERGY_VALUE_POSITION_CHAR 12
#define ENERGY_UNIT_POSITION_CHAR 18
#define ENERGY_UNIT "Wh"
#define ENERGY_CLEAR_MESSAGE "Time & Energy Reset"

#define DURATION_POSITION_ROW 3
#define DURATION_TITLE_POSITION_CHAR 0
#define DURATION_TITLE "D:"
#define DURATION_VALUE_POSITION_CHAR 3
#define DURATION_UNIT_POSITION_CHAR 16
#define DURATION_UNIT "DHMS"

#define seconds() (millis()/1000)

// commands
#define LCD_CLEARDISPLAY 0x01
#define LCD_RETURNHOME 0x02
#define LCD_ENTRYMODESET 0x04
#define LCD_DISPLAYCONTROL 0x08
#define LCD_CURSORSHIFT 0x10
#define LCD_FUNCTIONSET 0x20
#define LCD_SETCGRAMADDR 0x40
#define LCD_SETDDRAMADDR 0x80

// flags for display entry mode
#define LCD_ENTRYRIGHT 0x00
#define LCD_ENTRYLEFT 0x02
#define LCD_ENTRYSHIFTINCREMENT 0x01
#define LCD_ENTRYSHIFTDECREMENT 0x00

// flags for display on/off control
#define LCD_DISPLAYON 0x04
#define LCD_DISPLAYOFF 0x00
#define LCD_CURSORON 0x02
#define LCD_CURSOROFF 0x00
#define LCD_BLINKON 0x01
#define LCD_BLINKOFF 0x00

// flags for display/cursor shift
#define LCD_DISPLAYMOVE 0x08
#define LCD_CURSORMOVE 0x00
#define LCD_MOVERIGHT 0x04
#define LCD_MOVELEFT 0x00

// flags for function set
#define LCD_8BITMODE 0x10
#define LCD_4BITMODE 0x00
#define LCD_2LINE 0x08
#define LCD_1LINE 0x00
#define LCD_5x10DOTS 0x04
#define LCD_5x8DOTS 0x00

// flags for backlight control
#define LCD_BACKLIGHT 0x08
#define LCD_NOBACKLIGHT 0x00

#define En B00000100  // Enable bit
#define Rw B00000010  // Read/Write bit
#define Rs B00000001  // Register select bit

/**
 * This is the driver for the Liquid Crystal LCD displays that use the I2C bus.
 *
 * After creating an instance of this class, first call begin() before anything else.
 * The backlight is on by default, since that is the most likely operating mode in
 * most cases.
 */
class Robojax_Energy_Meter : public Print {
public:
	/**
	 * Constructor
	 *
	 * @param lcd_addr	I2C slave address of the LCD display. Most likely printed on the
	 *					LCD circuit board, or look in the supplied LCD documentation.
	 * @param lcd_cols	Number of columns your LCD display has.
	 * @param lcd_rows	Number of rows your LCD display has.
	 * @param charsize	The size in dots that the display has, use LCD_5x10DOTS or LCD_5x8DOTS.
	 */
	Robojax_Energy_Meter(uint8_t lcd_addr, uint8_t lcd_cols, uint8_t lcd_rows, uint8_t charsize = LCD_5x8DOTS);
	
	//Constructor for Current Measurement
	Robojax_Energy_Meter(uint8_t lcd_addr, uint8_t lcd_cols, uint8_t lcd_rows, 
							 uint8_t shuntInPin, uint8_t shuntCurrent, uint8_t shuntVoltage,
							 uint8_t correctionValue, uint8_t iteration, float voltageReference,
							 uint8_t bitResolution, float minimumCurrent, bool debugOnce,
						uint8_t charsize = LCD_5x8DOTS);
						
	//constructor for Energy Calculation
	Robojax_Energy_Meter(uint8_t lcd_addr, uint8_t lcd_cols, uint8_t lcd_rows, 
							 uint8_t shuntInPin, uint8_t shuntCurrent, uint8_t shuntVoltage,
							 uint8_t correctionValue, uint8_t iteration, float voltageReference,
							 uint8_t bitResolution, float minimumCurrent, bool debugOnce,
							 float ValueR1, float ValueR2,uint8_t voltageInPin, uint8_t pushButtonPin,
						uint8_t charsize = LCD_5x8DOTS);						

	float getCurrent();
	void printSerial();
	void printDebug();
	

	void showCurrent();
	void showVoltage();
	

    float getEnergy();
	void showEnergy();
	

    float getPower();
	void showPower();	
	void showDuration();
    String secondsToTime();
	void clearLine(uint8_t line);

	/**
	 * Set the LCD display in the correct begin state, must be called before anything else is done.
	 */
	void begin();

	 /**
	  * Remove all the characters currently shown. Next print/write operation will start
	  * from the first position on LCD display.
	  */
	void clear();

	/**
	 * Next print/write operation will will start from the first position on the LCD display.
	 */
	void home();

	 /**
	  * Do not show any characters on the LCD display. Backlight state will remain unchanged.
	  * Also all characters written on the display will return, when the display in enabled again.
	  */
	void noDisplay();

	/**
	 * Show the characters on the LCD display, this is the normal behaviour. This method should
	 * only be used after noDisplay() has been used.
	 */
	void display();

	/**
	 * Do not blink the cursor indicator.
	 */
	void noBlink();

	/**
	 * Start blinking the cursor indicator.
	 */
	void blink();

	/**
	 * Do not show a cursor indicator.
	 */
	void noCursor();

	/**
 	 * Show a cursor indicator, cursor can blink on not blink. Use the
	 * methods blink() and noBlink() for changing cursor blink.
	 */
	void cursor();

	void scrollDisplayLeft();
	void scrollDisplayRight();
	void printLeft();
	void printRight();
	void leftToRight();
	void rightToLeft();
	void shiftIncrement();
	void shiftDecrement();
	void noBacklight();
	void backlight();
	bool getBacklight();
	void autoscroll();
	void noAutoscroll();
	void createChar(uint8_t, uint8_t[]);
	void setCursor(uint8_t, uint8_t);
	virtual size_t write(uint8_t);
	void command(uint8_t);

	inline void blink_on() { blink(); }
	inline void blink_off() { noBlink(); }
	inline void cursor_on() { cursor(); }
	inline void cursor_off() { noCursor(); }

// Compatibility API function aliases
	void setBacklight(uint8_t new_val);				// alias for backlight() and nobacklight()
	void load_custom_character(uint8_t char_num, uint8_t *rows);	// alias for createChar()
	void printstr(const char[]);

private:
    void calculateEnergy();		
	void calculateCurrent();
	void calculatePower();	
    void readVoltage();
	void send(uint8_t, uint8_t);
	void write4bits(uint8_t);
	void expanderWrite(uint8_t);
	void pulseEnable(uint8_t);
	uint8_t _addr;
	uint8_t _displayfunction;
	uint8_t _displaycontrol;
	uint8_t _displaymode;
	uint8_t _cols;
	uint8_t _rows;
	uint8_t _charsize;
	uint8_t _backlightval;
	
	//current related values
   uint8_t _shuntInPin, _correctionValue, _iteration, _bitResolution, _currentValue; 
   float _voltageReference, _shuntCurrent, _shuntVoltage, _minimumCurrent;
   bool _debugOnce,_is_energy;
   //Energy  related
   uint8_t _voltageInPin, _pushButtonPin;
   float _inputVoltage, _ValueR2, _ValueR1, _maxVoltage, _energy;
   unsigned long _seconds, _power,_resetTime;
};

#endif // ROBOJAX_SHUNT_CURRENT_H
